@@ -4,7 +4,7 @@
 """
 ===============================================================================
 
-    Copyright (C) 2009-2018 Rudolf Cardinal (rudolf@pobox.com).
+    Original code copyright (C) 2009-2018 Rudolf Cardinal (rudolf@pobox.com).
 
     This file is part of cardinal_pythonlib.
 
@@ -22,9 +22,11 @@
 
 ===============================================================================
 
+**Functions to make it easy to serialize Python objects to/from JSON.**
+
 See ``notes_on_pickle_json.txt``.
 
-THE STANDARD PYTHON REPRESENTATION USED IS A DICTIONARY LIKE THIS:
+The standard Python representation used internally is a dictionary like this:
 
 .. code-block:: python
 
@@ -39,6 +41,7 @@ THE STANDARD PYTHON REPRESENTATION USED IS A DICTIONARY LIKE THIS:
     }
 
 We will call this an ``InitDict``.
+
 Sometimes positional arguments aren't necessary and it's convenient to work
 also with the simpler dictionary:
 
