@@ -42,6 +42,7 @@ from sqlalchemy.dialects import mssql, mysql
 from sqlalchemy.engine import Connection, Engine, ResultProxy
 from sqlalchemy.engine.interfaces import Dialect
 from sqlalchemy.engine.reflection import Inspector
+from sqlalchemy.dialects.mssql.base import TIMESTAMP as MSSQL_TIMESTAMP
 from sqlalchemy.schema import (Column, CreateColumn, DDL, MetaData, Index,
                                Sequence, Table)
 from sqlalchemy.sql import sqltypes, text
@@ -900,8 +901,8 @@ def convert_sqla_type_for_dialect(
     # -------------------------------------------------------------------------
     # TIMESTAMP
     # -------------------------------------------------------------------------
-    if (isinstance(coltype, sqltypes.TIMESTAMP) and to_mssql and
-            convert_mssql_timestamp):
+    is_mssql_timestamp = isinstance(coltype, MSSQL_TIMESTAMP)
+    if is_mssql_timestamp and to_mssql and convert_mssql_timestamp:
         # You cannot write explicitly to a TIMESTAMP field in SQL Server; it's
         # used for autogenerated values only.
         # - http://stackoverflow.com/questions/10262426/sql-server-cannot-insert-an-explicit-value-into-a-timestamp-column  # noqa
